*-- File: COMBO.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This UI file is designed to give some optional Combobox objects
*      which can be used either 'as is' or you can even subclass them.
*
*-- Programmers: 
*
*      Ken Mayer
*      Jim Sare (CIS: 71333,1030)
*      Romain Streiff (CIS: 71333,2147)
*      Bowen Moursund (CIS: 72662,436)
*      David Kanter (CIS: 73252, 662)
*
*-- History: 
*
*      12/23/1996 -- First version of this custom class file.
*
*-- Instantiation: 
*
*      set procedure to combo.cc additive
*      * create/modify a form, place an instance of one of
*      * these entryfields on the form as a normal entryfield ...
*
*-- Classes:
*      More details on each of these is contained in the commentary for
*      each class ...
*
*-- CodeCombo
*      Used with those tables that store a code, but you want to 
*      display a value in the combobox associated with the code.
*
*-- ComboDrop
*      Upon obtaining focus, this combobox drops down automatically,
*      without user need to click on down-arrow.
*
*----------------------------------------------------------------------

CLASS CodeCombo(f,c) OF COMBOBOX(f,c) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *     Custom class, combobox that handles codes and text
   *
   *-- Programmers: 
   *
   *      Romain Streiff (CIS: 71333,2147)
   *      Bowen Moursund (CIS: 72662,436)
   *
   *-- History: 
   *
   *      02/xx/1996 -- Original
   *      12/27/1996 -- Very minor modifications for the dUFLP library.
   *                    (and heavily updated documentation based on
   *                     messages from the VDBASE forum on Compuserve)
   *
   *-- Instantiation: 
   *
   *      set procedure to combo.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *
   *      Set the datasource to the field you are obtaining a value 
   *      from.
   *
   *      Set the datalink to the field in the table you wish to update
   *      (add, edit, whatever).
   *
   *      In the form's OnOpen (or the combobox object's OnOpen, but the
   *      form's is preferable), set the CodeField property:
   *         form.CodeCombo1.CodeField = "tableoralias->codefieldname"
   *
   *      ***********************************************************
   *      ** NOTE: If you need to use the _same_ table for both the 
   *      ** datasource and the datalink, you will need to open the 
   *      ** table twice in the query used for the form:
   *         use tablename order indextag
   *         use tablename order indextag again alias comboalias
   *      ** This is because the field being searched by the lookup()
   *      ** function, specified in the CodeField property cannot
   *      ** be in the main table -- this is because the record
   *      ** pointer gets moved ...
   *      ***********************************************************
   *
   *-- Example:
   *
   *     Assuming the CUSTOMER and ORDERS tables in the samples
   *     directory of Visual dBASE, you would want to create a
   *     query that opened the two tables:
   *
   *        use orders
   *        use customer order customer_n in select()
   *
   *     Placing an instance of this object on the form, set the
   *     datasource to:
   *
   *        datasource  "field customer->name"
   *     
   *     and the datalink:
   *
   *        datalink    "orders->customer_n"
   *
   *     In the form's OnOpen method, set:
   *
   *        form.CodeCombo1.CodeField = "customer->customer_n"
   *
   *     -------------------------------------------
   *     If you wanted to use the customer table only, you would need
   *     to do something along these lines:
   *
   *     Create a query that opens the table twice:
   *        use customer order customer_n
   *        use customer in select() again noupdate alias code
   *
   *     Place an instance of the object on the form, set the datasource:
   *
   *        datasource "field code->name"
   *
   *     set the datalink:
   *
   *        datalink "customer->customer_n"
   *
   *     And set, in the OnOpen method:
   *
   *        form.CodeCombo1.CodeField = "code->customer_n"
   *
   *-- Properties:
   *
   *--   CodeField
   *       Must be set by developer/programmer -- give fieldname
   *       that contains code (see instantiation and example above).
   *
   *--   form.CodeCombos
   *       Array used to display appropriate values (handled 
   *       automatically)
   *
   *--   CallFormOnNavigate
   *       Do you wish to use form's OnNavigate (defaults to .t.)?
   *
   *--   OriginalFormOnNavigate
   *       Stores form's original 'OnNavigate' method (handled
   *       automatically)
   *-------------------------------------------------------------------

   if type("form.CodeCombos") <> "A"
      form.CodeCombos = new array()
      this.CallFormOnNavigate = .t.   && set flag
      form.OriginalFormOnNavigate = form.OnNavigate
   else
      this.CallFormOnNavigate = .f.   && set flag
   endif

   form.CodeCombos.add(this)
   this.style = 2       && gotta use the listbox
   this.CodeField = ""  && must be set externally, maybe in
                        && form.OnOpen event handler
   this.onchange = class::SetLinkerValue

   PROCEDURE OnOpen
      form.OnNavigate = CodeCombo::OnNavigate
      this.PromptField = substr(this.datasource, ;
                                at(" ", this.datasource)+1)
      this.WorkArea = select(left(this.datalink, at(">", ;
                                this.datalink)-2))
      private cLinkerName, cLinkerObj, nLinker
      cLinkerName = this.name + "Linker"
      define entryfield &cLinkerName. of form property visible .f.,;
                  datalink this.datalink
      cLinkerObj = "form." + cLinkerName
      this.Linker = &cLinkerObj.
      this.datalink = ""
      class::SetPromptValue(this)
     
   PROCEDURE OnNavigate(nWorkArea)
      local n
      for n = form.CodeCombos.size to 1 step -1
          if form.CodeCombos[n].WorkArea = nWorkArea
             form.CodeCombos[n].SetPromptValue(form.CodeCombos[n])
          endif
          if form.CodeCombos[n].CallFormOnNavigate .and. ;
                        .not. empty(form.OriginalFormOnNavigate)
             form.OriginalFormOnNavigate(nWorkArea)
          endif
      endfor
      
   PROCEDURE SetPromptValue
      parameter oCodeCombo
      local fpOnNavigate
      fpOnNavigate = form.OnNavigate
      form.OnNavigate = {;}
      private cLookInField, cReturnField, cLookFor
      cLookInField = oCodeCombo.CodeField
      cReturnField = oCodeCombo.PromptField
      cLookFor = oCodeCombo.Linker.value
      oCodeCombo.value = lookup(&cReturnField., cLookFor, ;
                                &cLookInField.)
      form.OnNavigate = fpOnNavigate
 
   PROCEDURE SetLinkerValue
      private cLookInField, cReturnField, cLookFor
      cReturnField = this.CodeField
      cLookInField = this.PromptField
      cLookFor = this.value
      this.Linker.value = lookup(&cReturnField., cLookFor, ;
                                 &cLookInField.)

ENDCLASS
*-- End of Class: CodeCombo

CLASS ComboDrop(FormObj,Name) OF COMBOBOX(FormObj,Name) Custom
   *-------------------------------------------------------------------
   *-- Description:
   *
   *     Creates a combobox that immediately drops down when it
   *     gets focus.
   *
   *-- Programmer: 
   *
   *      Jim Sare (CIS: 71333,1030)
   *      David Kanter (CIS: 73252,662)
   *
   *-- History: 
   *
   *      10/15/1995 -- Original
   *      11/29/1996 -- Very minor modifications for the dUFLP library.
   *      12/26/1996 -- Added OnLeftMouseDown method so box drops when 
   *                    clicked
   *
   *-- Instantiation: 
   *
   *      set procedure to combo.cc additive
   *      * create/modify a form, place an instance of this entryfield
   *      * on the form, and datalink as usual.
   *
   *-- Properties:
   *
   *--   DropOnGotFocus
   *       Logical -- defaults to true. If you wish to use this in its
   *       default configuration (i.e., don't drop down when the user
   *       tabs into the combobox) set it to false.
   *-------------------------------------------------------------------

   if type("SendMessage") # "FP"
      extern CLONG SendMessage( CHANDLE, CINT, CWORD, CLONG ) USER
   endif
   #ifndef WM_USER
      #define WM_USER HToI("400")
   #endif
   #ifndef CB_SHOWDROPDOWN
      #define CB_SHOWDROPDOWN WM_USER + 15
   #endif
   #ifndef FALSE
      #define FALSE 0
   #endif
   #ifndef TRUE
      #define TRUE  1
   #endif
   
   this.DropOnGotFocus = .t.  && Drop when user tabs into ComboBox
   this.Style          = 2    && DropDownList
   
   PROCEDURE OnGotFocus
      if this.DropOnGotFocus
         this.Drop()
      endif
   
   PROCEDURE Drop
      SendMessage( this.hWND, CB_SHOWDROPDOWN, TRUE, 0 )
   
   PROCEDURE PullUp
      SendMessage( this.hWND, CB_SHOWDROPDOWN, FALSE, 0 )

   Procedure OnLeftMouseDown
      class::OnGotFocus()
      
ENDCLASS
*-- End of Class: ComboDrop

*----------------------------------------------------------------------
*-- End of File: COMBO.CC
*----------------------------------------------------------------------
