*----------------------------------------------------------------------
*
* HLDBXMPL.PRG -- Examples for the use of HLDBASE.CC and HLDBASE.DLL
*
*----------------------------------------------------------------------
*
*-- Description:
*
*      example code
*
*-- Programmer:
*
*      Marcello Perathoner (CIS: 100752,3200)
*
*-- History:
*
*      12/11/1996 -- original version
*
*-- Call
*
*      do hldbxmpl
*
*---------------------------------------------------------------------

set procedure to hldbase.cc additive
local hl  ; hl  = new HLDBase () && create object
local out ; out = new regs ()	 && create output regs

* we start with something very simple

************************************************************************
?
? "get drive allocation information"
?
***********************************************************************

local in
in = new regs ()	&& use a new one for every call !!!
in.ah = 54		&& dos function nr
in.dl = 3	        && 0=cur, 1=A, ...

hl.intdos (in, out)

*
* registers ax, bx, cx, dx contain unsigned integers
*

? "sectors per cluster ", out.ax
? "available clusters  ", out.bx
? "bytes per sector    ", out.cx
? "clusters per drive  ", out.dx
? "free bytes on drive ", out.cx * out.ax * out.bx, diskspace (3)
? "total bytes on drive", out.cx * out.ax * out.dx

***********************************************************************
?
? "IOCTL - get volume serial nr"
?
***********************************************************************

* This function requires that we provide a buffer where DOS can
* store the requested information.

in      = new regs ()	&& use a new one for every call !!!
in.ax   = htoi ("440D") && function 44h subfunction 0Dh
in.bl   = 3		&& 0=cur, 1=A
in.cx   = htoi ("0866") && more subfunction numbers
in.dsdx = hlcalloc (25) && allocate a string of 25 * chr (0) as buffer

hl.intdos (in, out)

* We must use in.dsdx again because it is there that DOS stored the
* requested information. The output registers dont know that they are
* holding a pointer to data. In fact they dont even have a property
* dsdx.

* hlgets32 gets a _S_igned _32_ bits out of the string starting
* at the requested position and stores them into a numeric dBase
* variable. We should really get an unsigned 32 bits, but the internal
* dBase format for integer numeric data is signed.

? "volume serial nr "  + itoh (hlgets32 (in.dsdx, 3))
? "volume label     '" + trim (substr (in.dsdx, 7, 11)) + "'"
? "filesystem type  '" + trim (substr (in.dsdx, 18, 8)) + "'"

***********************************************************************
?
? "get drive parameter block"
?
***********************************************************************

* This function returns a pointer to the drive parameter block
* in dx:bx. To access ds:bx we must peek into memory.

in = new regs ()	&& use a new one for every call !!!
in.ah = htoi ("32")	&& function nr
in.dl = 3	        && 0=cur, 1=A

hl.intdos (in, out)

* Now ds and bx point to the drive parameter block. We must copy
* it into a string. hlpeek copies up to 32766 bytes into a string
* starting at seg, off.

cDPB = hl.peek (out.ds, out.bx, 33)  && copy 33 bytes at ds:bx to string

* hlgetu8 gets an _U_nsigned 8 bits out of the string
* hlgetu16 gets an _U_nsigned 16 bits out of the string

? "nr of reserved sectors           ", hl.getu16 (cDPB, 7)
? "nr of FATs                       ", hl.getu8  (cDPB, 9)
? "nr of sectors per FAT            ", hl.getu8  (cDPB, 16)
? "nr of first directory sector     ", hl.getu16 (cDPB, 18)
? "nr of root directory entries     ", hl.getu16 (cDPB, 10)
? "nr of first sector with user data", hl.getu16 (cDPB, 12)

***********************************************************************
?
? "get com / lpt base addresses"
?
***********************************************************************

* This "simply" peeks at address 0040:0000. Since we cannot use
* real mode selectors, we have to create an equivalent protected 
* mode selector.

* allocate a read/write selector to segment 0x0040

sel = hl.AllocateRealModeSelector (htoi ("0040"))

? "com1 base address", itoh (hl.peeku16 (sel,  0))
? "com2 base address", itoh (hl.peeku16 (sel,  2))
? "com3 base address", itoh (hl.peeku16 (sel,  4))
? "com4 base address", itoh (hl.peeku16 (sel,  6))
? "lpt1 base address", itoh (hl.peeku16 (sel,  8))
? "lpt2 base address", itoh (hl.peeku16 (sel, 10))
? "lpt3 base address", itoh (hl.peeku16 (sel, 12))

hl.FreeRealModeSelector (sel)  && dont forget !
hl.Release ()

* Have fun. But download ralf browns interrupt list first.
*
* Marcello



