*-- File: HOLIDAY.CC

CLASS Holiday
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains many useful date-handling methods. For other
*      date and 'Hebrew dates', check the files: "DATE.CC" and 
*      "HEBREW.CC"
*
*-- Programmers: 
*
*      Ken Mayer
*      Jay Parsons (CIS 72662,1302)
*
*-- History: 
*
*      11/25/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to holiday.cc additive
*      _app.Holiday = new Holiday()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*
*--    Init()
*        Makes sure other classes needed are available
*
*--    Release()
*        Releases this object from memory
*
*--    EasterDay()
*        Returns date of Easter for year
*
*--    FWDoM()
*        First Working Day of Month
*
*--    HoliCount()
*        Number of holidays from first of year to date given
*
*--    Holiday()
*        Date of specific holiday
*
*--    HolXFer()
*        Actual date of celebration for specific holiday
*
*--    LWDoM()
*        Last Working Day of Month
*
*--    NoWeekNd()
*        Moves date off weekend to Monday, if it falls on Sat/Sunday
*
*--    SWorkDays()
*        Number of working days between two dates, excluding
*        first date, including second.
*
*--    WDiF()
*        Working Days in Future
*
*--    WeekDays()
*        Working days in range of dates
*
*--    XWorked()
*        Number of work days in a one-week period, starting at 
*        nth day ...
*
*      *********************************************************
*      *** The following methods use the table: MAKEHOLS.DBF ***
*      *** And/or a table HOLSyyyy.DBF (year)                ***
*      *********************************************************
*
*--    AddaHol()
*        Add a holiday to MAKEHOLS.DBF
*
*--    MakeHols()
*        Create HOLSyyyy.DBF table
*
*--    WorkDays()
*        Number of work days between range of dates, excluding first,
*        including second.
*
*-- Dependencies:
*
*      DATEEX.CC
*      TIME.CC
*      HEBREW.CC
*      DUFLP.H
*----------------------------------------------------------------------

   CLASS::Init()
   
   FUNCTION Init
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/09/1996
   *-- Notes.......: Ensures that routines needed are available
   *--               It is required that this be used (see instructions
   *--               above) for this class to function properly.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/09/1996 -- Original Code
   *-- Calls.......: date.cc; time.cc; hebrew.cc
   *-- Usage.......: holiday.Init()
   *-- Example.....: _app.Holiday.Init()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------

      PROTECT DateProc, TimeProc, HebrewProc
      
      if .not. SetProc( "DATEEX.CC" )
         set procedure to dateex.cc   additive
         this.DateProc = .t.
      else
         this.DateProc = .f.
      endif
      this.Date        = new DateEx()
      this.Date.parent = this

      if .not. SetProc( "TIME.CC" )
         set procedure to time.cc   additive
         this.TimeProc = .t.
      else
         this.TimeProc = .f.
      endif
      this.Time          = new Time()
      this.Time.parent   = this

      if .not. SetProc( "HEBREW.CC" )
         set procedure to hebrew.cc additive
         this.HebrewProc = .t.
      else
         this.HebrewProc = .f.
      endif
      this.Hebrew        = new Hebrew()
      this.Hebrew.parent = this
      
   RETURN NULL
   
   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: holiday.Release()
   *-- Example.....: _app.Holiday.Release()
   *--               _app.Holiday = NULL  && not required, good idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------
      if this.DateProc
         close procedure dateex.cc
      endif
      if this.TimeProc
         close procedure time.cc
      endif
      if this.HebrewProc
         close procedure hebrew.cc
      endif
      release object this
   
   RETURN NULL
   *-- EoM: Release()

   FUNCTION EasterDay( nYear )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS 72662,1302)
   *-- Date........: 04/22/1994
   *-- Notes.......: Returns date of Easter for given year after 1582.
   *--               This gives the date of Easter as celebrated by 
   *--               Western churches.  The algorithm is from Example 
   *--               1.3.2.14 of Volume I of "The Art of Computer 
   *--               Programming", 2nd Edition, Addison-Wesley, 
   *--               Reading, MA, 1973, by Donald Knuth, who attributes
   *--               it to Aloysius Lilius of Naples and Christopher 
   *--               Clavius of Germany, both floruit 1582.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/18/1991 - original function.
   *--               04/22/1992 - Jay Parsons - Notes expanded.
   *--               11/20/1992 - David Love -  Added the private 
   *--               variable lYear.
   *--               12/03/1992 - Jay Parsons - renamed lYear and 
   *--               dPascMoon
   *--               08/02/1993 - Jay Parsons - changed iif to if/endif
   *--               04/22/1994 - Jay Parsons - made international
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.EasterDay(<nYear>)
   *-- Example.....: _app.Holiday.EasterDay(91)
   *-- Returns.....: Date (in dBASE date format) of Easter
   *-- Parameters..: nYear  =  Numeric form of year - YYYY or YY format
   *-------------------------------------------------------------------
       
      local nYr, nGolden, nCentury, nNoLeap
      local nMoonOrbit, nEPact, cDateSet,dPascMoon, dReturn
      
      *-- deal with two digit year ...
      
      nYr = nYear
      if nYr < 100
         nYr = nYr + 100 * int( year( date() ) / 100 )
      endif
    
      *-- the "Golden number" of a year, used in many ecclesiastical
      *-- computations
      nGolden    = 1 + mod( nYr, 19 )
      *-- number of the century
      nCentury   = floor( nYr / 100 ) + 1
      *-- adjustment for non-leap century years 1700, 1800, 1900, etc.
      nNoLeap    = floor( 3 * nCentury / 4 ) - 12
      *-- adjustment for perturbations of orbit of the moon
      nMoonOrbit = floor( ( 8 * nCentury + 5 ) / 25 ) - 5
      *-- the Epact is the day() in January of the first full moon of
      *-- the year
      nEpact     = mod( 11 * nGolden + nMoonOrbit - nNoLeap ;
                            + 20, 30 )
      if nEpact  = 24 .or. ( nEPact = 25 .and. nGolden > 11 )
         nEpact  = nEpact + 1
      endif
      *-- the pascal moon is the first full moon on or after the spring
      *-- equinox
      cDateSet   = set("DATE")
      set date AMERICAN
      dPascMoon  = ctod( "03/21/" + str( nYr ) ) ;
                      + mod( 53 - nEPact, 30 )
      set date &cDateSet.
      *-- and Easter is the Sunday after the pascal moon
      dReturn    = dPascMoon + 8 - dow( dPascMoon )
      
   RETURN dReturn
   *-- EoF: EasterDay()

   FUNCTION FWDoM( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 02/25/1992
   *-- Notes.......: First Working Date of the Month -- date of the 
   *--               first day of the month that is not Saturday or 
   *--               Sunday.
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 02/25/1992 - Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.FWDoM(<dDate>)
   *-- Example.....: ? _app.Holiday.FWDoM(DATE())
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *-------------------------------------------------------------------

      local dReturn, nDay
      
      dReturn = dDate - day( dDate ) + 1
      nDay = dow( dReturn )
      
   RETURN dReturn + iif( nDay = 7, 2, iif( nDay = 1, 1, 0 ) )
   *-- EoF: FWDoM()

   FUNCTION HoliCount( dDate, cHols )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 06/19/1993
   *-- Notes.......: Returns number of holidays that fall in the 
   *--               portion of a calendar year ending with a given 
   *--               date.  Always includes New Year's Day, transferred 
   *--               if required to the 2d or 3d, and July 4 and Xmas, 
   *--               each transferred if required to the nearest day 
   *--               not Saturday or Sunday. Other holidays are 
   *--               included ( but not transferred ) if their codes 
   *--               for the Holiday() function are included in
   *--               the cHols argument. Does not include the numbers 
   *--               of Saturdays and Sundays in the value returned.
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/19/1993 - original function.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: date.Annivrsry()       
   *--               holiday.Holiday()      
   *--               date.FDoY() 
   *--               holiday.HolXFer() 
   *-- Usage.......: holiday.HoliCount(<dDate>,<cHols>)
   *-- Example.....: _app.Holiday.HoliCount( {06/19/93}, "PMLCT" )
   *-- Returns.....: Number of holidays
   *-- Parameters..: dDate     -- date ending the period
   *--               cHols     -- list of holidays for function 
   *--                            Holiday()
   *-------------------------------------------------------------------

      local dSt, dThis, nH, nHols
      
      nH = 1
      nHols = 0
      dSt = this.Date.FDoY( dDate )             && first day of our year
      do while nH < len( cHols ) + 4
         do case
            case nH <= len( cHols )      && a day in cHols
                 dThis = CLASS::Holiday( year( dSt ), ;
                            substr( cHols, nH, 1 ) )
            case nH = len( cHols )       && New Year's Day
                 dThis = CLASS::HolXFer( dSt, "17", "F" )
            case nH = len( cHols ) + 2   && Independence Day
                 dThis = ctod( "07/04/" + str( year( dDate ), 4 ) )
                 dThis = CLASS::HolXFer( dThis, "17", "Y" )
            case nH = len( cHols ) + 3   && Christmas
                 dThis = ctod( "12/25/" + str( year( dDate ), 4 ) )
                 dThis = CLASS::HolXFer( dThis, "17", "Y" )
         endcase
         if this.Date.Annivrsry( dThis, dSt, dDate )
            nHols = nHols + 1
         endif
         nH = nH + 1
      enddo

   RETURN nHols
   *-- EoF: HoliCount()

   FUNCTION Holiday( nYear, cCode )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 70160,340)
   *-- Date........: 04/22/1994
   *-- Notes.......: Returns the date of a specified "floating" holiday
   *--               (using table below) for given year.
   *--               Name                 Code
   *--               President's Day      P
   *--               Daylight saving time D
   *--               Memorial Day         M
   *--               Labor Day            L
   *--               Columbus Day         C
   *--               resume Standard time S
   *--               Election Day         E
   *--               Thanksgiving         T
   *--               Advent (1st Sunday)  A
   *--                     To add additional days, specify the month 
   *--               and date the holiday may first possibly occur, 
   *--               MM/DD/, as cFirst, and the dow() on which it 
   *--               occurs as nDow. This will work only if the holiday 
   *--               occurs on a fixed day of the week in a fixed 
   *--               seven-day period.
   *--
   *--               WARNING: This function assumes set("DATE") is 
   *--               AMERICAN. To use it with different settings, store 
   *--               the day and month of the holidays in the chosen 
   *--               date format and change the ctod() function in the 
   *--               line in which dBaseDate is calculated to match 
   *--               that format.
   *--
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 11/01/1991 - original function.
   *--               11/15/1991 - Ken Mayer - takes a code and year -- 
   *--               basically simplified the use of the function.
   *--               04/22/1992 - Jay Parsons - added 'D' and 'S' 
   *--               options (daylight saving time and return to 
   *--               standard)
   *--               08/03/1993 - Jay Parsons - removed change to 
   *--               parameter
   *--               04/22/1994 - Jay Parsons - added note about date 
   *--               set()
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.Holiday(<nYear>, <cCode>)
   *-- Example.....: ? _app.Holiday.Holiday(92,"P") 
   *--                            && date of President's day, 1992
   *-- Returns.....: Date of specified holiday ...
   *-- Parameters..: nYear = Year you need the holiday date for ...
   *--               cCode = one of the codes above for specific 
   *--                       holiday
   *-------------------------------------------------------------------
       
      local cC, cYear, cFirst, nDoW, dReturn, dBaseDate
         
      cC = upper( cCode )
      cYear = ltrim( str( nYear ) )
      do case
         case cC = "P"    && President's day (3rd Mon Feb)
            cFirst = "02/15/"
            nDoW   = 2
         case cC = "D"    && Daylight time U.S. (1st Sun Apr)
            cFirst = "04/01/"
            nDoW   = 1
         case cC = "M"    && Memorial day  (last Mon May)
            cFirst = "05/25/"
            nDoW   = 2
         case cC = "L"    && Labor day  (1st Mon Sep)
            cFirst = "09/01/"
            nDoW   = 2
         case cC = "C"    && Columbus Day  (2nd Mon Oct)
            cFirst = "10/08/"
            nDoW   = 2
         case cC = "S"    && Standard Time U.S. (last Sun Oct)
            cFirst = "10/25/"
            nDoW = 1
         case cC = "E"    && Election Day  (1st Tues Nov, but
                          && not Nov 1)
            cFirst = "11/02/"
            nDoW   = 3
         case cC = "T"    && Thanksgiving (4th Thurs Nov)
            cFirst = "11/22/"
            nDoW   = 5
         case cC = "A"    && 1st Sunday of Advent (Sunday
                          && closest to Nov 30)
            cFirst = "11/27/"
            nDoW   = 1
         otherwise
            RETURN {}   && return blank date for error
      endcase
      * start with first day the holiday may occur and use as the base
      * date that one of the possible dates that is a week ahead less
      * the dow() of the holiday--we later add the dow() back.
      dBasedate = ctod( cFirst + cYear) + 7 - nDow
      * Back up to the previous Saturday and add the dow().
      dReturn = dBaseDate - dow( dBaseDate ) + nDoW
   
   RETURN dReturn
   *-- EoF: Holiday()

   FUNCTION HolXFer( dDate, cDoWs, cDir )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 06/19/1993
   *-- Notes.......: Returns date of celebration of a holiday.  This 
   *--               will be the date given unless it falls on one of 
   *--               the days of the week whose dow() values are given 
   *--               as chars in the cDows parameter, in which case it 
   *--               will be transferred to the first available date 
   *--               without such a dow() either forward, backward or 
   *--               to the nearest open date depending on the third 
   *--               cDir parameter being "F", "B" or "Y".
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/19/1993 - original function.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.HolXfer(<dDate>,<cDows>,<cDir>)
   *-- Example.....: ?_app.Holiday.HolXfer( {12/25/93}, "17"," ")
   *-- Returns.....: Date holiday is celebrated
   *-- Parameters..: dDate     -- date of the actual holiday.
   *--               cDoWs     -- string of all dow() values from which 
   *--                            it must be transferred if it falls on
   *--                            one.
   *--               cDir      -- character "F" for forward transfer 
   *--                            only, "B" for backward transfer only 
   *--                            or "Y" for either way, forward being 
   *--                            preferred (default).
   *-------------------------------------------------------------------------------
      
      local nDir, dRet, nMoves
      
      dRet = dDate
      do case
         case upper( cDir ) = "F"
              nDir = 1
         case upper( cDir ) = "B"
              nDir = -1
         case upper( cDir ) = "Y"
              nDir = 0
         otherwise
              RETURN dRet
      endcase
      nMoves = 1
      do while str( dow( dRet ), 1 ) $ cDoWs
         if nDir # 0
            dRet = dRet + nDir
         else
            if mod( nMoves, 2 ) = 1
               dRet = dRet + nMoves
            else
               dRet = dRet - nMoves
            endif
            nMoves = nMoves + 1
         endif
      enddo

   RETURN dRet
   *-- EoF: HolXFer()

   FUNCTION LWDoM( dDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 02/25/1992
   *-- Notes.......: Last Working Day of the Month -- returns last date
   *--               of the month not Saturday or Sunday. 
   *-- Written for.: dBASE IV, 1.1
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Rev. History: 02/25/1992 - Original Release
   *-- Calls.......: date.LDoM()
   *-- Usage.......: holiday.LWDoM(<dDate>)
   *-- Example.....: ? _app.Holiday.LWDoM(DATE())
   *-- Returns.....: dBASE Date
   *-- Parameters..: dDate  -- date to work from ...
   *-------------------------------------------------------------------
      
      local dReturn, nDay

      dReturn = this.Date.LDoM( dDate )
      nDay = dow( dReturn )
      
   RETURN dReturn - iif( nDay = 7, 1, iif( nDay = 1, 2, 0 ) )
   *-- EoF: LWDoM()

   FUNCTION NoWeekNd( xDate )
   *-------------------------------------------------------------------
   *-- Programmer..: Malcolm C. Rubel
   *-- Date........: 04/01/1994
   *-- Notes.......: This was lifted from the April 1994 issue of
   *--               Data Based Advisor, the "dBASE Developer" column,
   *--               written by Mac.
   *--               This routine simply pushes a date off a weekend
   *--               (to Monday) if it ends up there.  
   *-- Written for.: dBASE IV, 1.5+
   *-- Rev. History: 04/01/1994 -- Published
   *--               04/15/1994 -- Adapted for dUFLP (KJM).
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.NoWeekNd(<xDate>)
   *-- Example.....: dDue = _app.Holiday.NoWeekNd(dDue)
   *-- Returns.....: Date/Character
   *-- Parameters..: xDate = EITHER Character _or_ Date type in 
   *--                       standard dBASE date format.
   *-------------------------------------------------------------------
      
      local dDate, dReturn
      
      *-- if no parameter passed, use current system date
      dDate = iif( pcount() = 0, date(), xDate)
      
      if type("xDate") = "C"     && if passed as a character
         dDate = ctod(xDate)     && convert to dBASE date format
         lChar = .t.             && mark here as character
      else
         lChar = .f.
      endif
      
      if .not. isblank(dDate)
      
         nDow = dow(dDate)      && get the day of week
         do case
            case nDow = 7            && it's a Saturday
                 dReturn = dDate+2   &&   add two
            case nDow = 1            && it's a Sunday
                 dReturn = dDate+1   &&   add one
            otherwise
                 dReturn = dDate
         endcase
         
      else
         
         dReturn = {}         && return a blank date

      endif
       
   RETURN iif( lChar, dtoc(dReturn), dReturn )
   *-- EoF: NoWeekNd()

   FUNCTION SWorkDays( dDate1, dDate2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 70160,340) 
   *-- Date........: 04/22/1994
   *-- Notes.......: Returns workdays between two dates, excluding the
   *--               first but including the last.
   *--                     This function is a shortcut that works only
   *--               if all of the following assumptions are true:
   *--                     1) Workdays are Monday through Friday.
   *--                     2) Holidays are: New Year's Day   ( 1/1 )
   *--                                      Independence Day ( 7/4 )
   *--                                      Christmas Day    ( 12/25 )
   *--                        and any selection of floating holidays 
   *--                        found in the Holiday() function, provided 
   *--                        their letters are inserted in "cHols" in 
   *--                        this function.  It can be made an 
   *--                        additional parameter if desired.
   *--                     3) If any holiday falls on Saturday or 
   *--                        Sunday, it is observed on the closest 
   *--                        workday, except New Year's Day which is 
   *--                        observed on the next workday.
   *--               For a more sophisticated treatment of holidays, 
   *--               see the methods MakeHols() and WorkDays() in 
   *--               this class.
   *--
   *--               WARNING: Because this function uses U. S. 
   *--               holidays, it also assumes set("DATE") is AMERICAN.
   *--               If adapting it to use with another date format, 
   *--               you may have to change the line containing a call 
   *--               to ctod() in the do loop.
   *--
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/19/1993 - original function.
   *--               04/22/1994 - Jay Parsons - added note on date 
   *--                            set()
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: holiday.HoliCount()
   *--               date.IsLeap()
   *--               date.FDoY() 
   *--               holiday.XWorked() 
   *--               holiday.WeekDays()
   *-- Usage.......: holiday.SWorkDays(<dDate1>,<dDate2>)
   *-- Example.....: ? _app.Holiday.SWorkDays(date(),{08/27/94})
   *-- Returns.....: Number of workdays in the period between the 
   *--               dates, excluding the first and including the last,
   *--               or -1 for error.
   *-- Parameters..: dDate1, dDate2 -- dates limiting the period.
   *-------------------------------------------------------------------
      
      local dStart, dEnd, nWork, nYr, cHols

      cHols = "PMLCT"       && codes for five floating holidays
                            && President's Day, Memorial Day
                            && Labor Day, Columbus Day, Thanksgiving
      * deal with improper arguments
      if type("dDate1") # "D" .or. type("dDate2") # "D"
         RETURN -1
      endif
      
      * and reversal of order
      dStart = min( dDate1, dDate2 )
      dEnd = max( dDate1, dDate2 )
      
      nWork = 0
      * Full years contain 52 weeks of 5 workdays each, less all the
      * holidays, plus one day the same day of the week as the
      * starting date of the year, and in leap years one more day.
      * We deal with calendar years and two stub periods to avoid the
      * possibility that a period of one year measured from the
      * starting date may contain more or less than a full year of
      * holidays--a year from January 2 to the next January 2 may
      * contain 0, 1 or 2 Leap Year holidays.  52 * 5 = 260, 260 -
      * 3 holidays always included = 257.
      nYr = year( dStart )     && start at 1/1 of start year
      do while year( dEnd ) > nYr
         nWork = nWork + 257 - len( cHols )
         nWork = nWork + ;
                 CLASS::XWorked( iif( this.Date.IsLeap( nYr ), 2, 1 ),;
                 dow( ctod( "01/01/" + str( nYr, 4 ) ) ) )
         nYr = nYr + 1
      enddo

      * now adjust for the weekdays of the initial year through the
      * start date
      nWork = nWork ;
                 - WeekDays( this.Date.FDoY( dStart ) - 1, dStart )
      * and the holidays in that stub period
      nWork = nWork + CLASS::HoliCount( dStart, cHols )
      
      * and for the weekdays of the final partial year
      nWork = nWork ;
                 + WeekDays( this.Date.FDoY( dEnd ) - 1, dEnd )
      * and the holidays in that stub period
      nWork = nWork - CLASS::HoliCount( dEnd, cHols )
      
   RETURN nWork
   *-- EoF: SWorkDays()

   FUNCTION WDiF( dStart, nWDays )
   *-------------------------------------------------------------------
   *-- Programmer..: Martin Leon (HMAN)
   *-- Date........: 12/12/1991
   *-- Notes.......: Returns the date of the last working day in the 
   *--               period that includes the given number of working 
   *--               days beyond the given starting date. You may 
   *--               optionally use a table HOLIDAYS.DBF with this 
   *--               structure, and one record for each holiday not 
   *--               falling on a weekend: 
   *--                 Field name  Field type  MDX?
   *--                 HOLIDATE      Date       Y
   *--               Once the UDF has been run, the database is left 
   *--               open in whatever work area it was opened.  If 
   *--               another database was in use at the time of calling 
   *--               the UDF, it becomes the active database after the
   *--               UDF is done. The reason for leaving the database 
   *--               open is that this speeds up the process when you 
   *--               call on the UDF several times in a row.
   *--               To ensure that holidays are working properly, 
   *--               there are 3 assumptions made by this function, 
   *--               and all must be true.  These are:
   *--                     1) WDIF() assumes that your holidays 
   *--                        database has an index tag on the 
   *--                        HOLIDATE field,
   *--                     2) there are no duplicate entries, and
   *--                     3) none of the holidays in the database 
   *--                        falls on a weekend date. A simple method 
   *--                        for insuring the last is:
   *--                         USE Holidays
   *--                         DELETE ALL FOR dow( Holidate ) = 7 ;
   *--                               .or. dow( Holidate ) = 1
   *--                         PACK
   *--               If you do not have a Holidays database, this 
   *--               function will work fine but will count all days 
   *--               that are Monday through Friday as working days
   *-- Written for.: dBASE IV, 1.1
   *-- Rev. History: 12/12/1991 -- Original Release
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.WDIF(<dStart>,<nWDays>)
   *-- Example.....: ? _app.Holiday.WDiF(date(),10)
   *-- Returns.....: dBASE date
   *-- Parameters..: dStart  =  Date to start counting from
   *--               nWDays   =  Number of working days in the future 
   *-------------------------------------------------------------------
      
      local nWeeks, nN, nXtraDays, nHDays, dReturn, cNear, ;
            cAlias, dTemp
      
      store 0 to nWeeks, nN, nHDays, nXtraDays
      store {} to dReturn, dTemp
      store "" to cNear, cAlias
      cNear = set("NEAR")
      
      if nWDays = 0
         RETURN 0
      endif
      
      if type("dStart") + type("nWDays") # "DN"
         RETURN -1
      endif
      
      *-- Rough guestimate of future date within a week
      nWeeks = int( nWDays / 5 )
      dReturn = dStart + ( nWeeks * 7 )
      
      *-- Left over number of days from integer division above
      nXtraDays = mod( nWDays, 5 )
      
      *-- Check to see if Holidays database is already in use.  This
      *-- is done so that we don't have to close and open the database
      *-- for every call to this UDF. The first call opens it and
      *-- subsequent calls select it as needed.
         
      *-- Check all work areas for holidays database, starting with
      *-- work area 10 since this is most likely where it was opened
      *-- the first time.
      nN = 10
      do while .not. "HOLIDAYS" $ alias( nN )
         nN = nN - 1
         if nN = 0
            exit
         endif
      enddo
      *-- If it is open, store current alias name and select holidays
      *-- database.
      if nN # 0
         cAlias = alias()
         select ( alias( nN ) )
      else
         *-- If it isn't the currently selected database,
         *-- make sure it exists and use it and select it.
         if file( "HOLIDAYS.DBF" )
            cAlias = alias()
            use Holidays order Holidate in select()
            select Holidays
         endif
      endif
      *-- If it's active now ...
      if alias() = "HOLIDAYS"
         *-- make sure it's in Holidate order, and ...
         if order() # "HOLIDATE"
            set order to Holidate
         endif
         set near on
         *-- count all records in holiday database that fall within
         *-- the range of the starting date and the rough guestimate
         *-- date.
         seek dStart
         *-- don't count starting day if it's in Holidays database.
         if dStart = Holidate
            skip
         endif
         scan while dReturn >= Holidate
              nHDays = nHDays + 1
         endscan
         set near off
      endif
   
      *-- Add holidays to "left over" days from original guestimate
      nXtraDays = nXtraDays + nHDays
      
      *-- Add extra days one day at a time to the guestimate,
      *-- skipping over holidays and weekends.
      
      do while nXtraDays > 0
         dReturn = dReturn + 1
         if alias() = "HOLIDAYS"
            if seek( dReturn )
               loop
            endif
         endif
         if dow( dReturn ) = 7 .or. dow( dReturn ) = 1
            loop
         endif
         nXtraDays = nXtraDays - 1
      enddo
      
      *-- If return date falls on Saturday or Sunday, "re-wind" to
      *-- Friday.
      dReturn = dReturn - iif( dow( dReturn ) = 7, 1, ;
                iif( dow( dReturn) = 1, 2, 0 ) )
      
      *-- If another database was origally in use, make it the active
      *-- database again.
      if "" # cAlias
         select ( cAlias )
      endif
      *-- set NEAR back to what it was orginally.
      set near &cNear.

   RETURN dReturn
   *-- EoF: WDiF()

   FUNCTION WeekDays( dDate1, dDate2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 06/19/1993
   *-- Notes.......: Returns number of days that are not Saturdays or
   *--               Sundays within a range of dates.  Excludes 
   *--               beginning date and includes ending date.  Range 
   *--               must be less than one year.  See "SWorkDays()" 
   *--               for method of calculating weekdays in a number 
   *--               of full years.
   *--               Does not consider holidays.
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/19/1993 - original function.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: date.Age2
   *--               holiday.XWorked()
   *-- Usage.......: holiday.WeekDays(<dDate1>,<dDate2>)
   *-- Example.....: _app.Holiday.WeekDays( {06/19/93},{12/31/93} )
   *-- Returns.....: Number of weekdays in the period between the 
   *--               dates, excluding the first and excluding the last,
   *--               or -1 for error.
   *-- Parameters..: dDate1    -- starting date.
   *--               dDate2    -- ending date.
   *-------------------------------------------------------------------
      
      local dStart, dEnd, nWork

      * deal with improper arguments
      if type("dDate1") # "D" .or. type("dDate2") # "D"
         RETURN -1
      endif
      
      * reversal of order
      dStart = min( dDate1, dDate2 )
      dEnd = max( dDate1, dDate2 )
      
      * and periods of a year or more
      if this.Date.Age2( dStart + 1, dEnd ) > 0
         RETURN -1
      endif

      * 5 days per week for the whole weeks
      nWork = 5 * int( ( dEnd - dStart ) / 7 )
      
      * and the stub days ( revised to accommodate a mod() bug )
      nWork = nWork ;
                 + Xworked( mod( fixed( dEnd - dStart ), 7 ), ;
                            dow( dStart + 1 ) )
   RETURN nWork
   *-- Eof: WeekDays()

   FUNCTION XWorked( nXtra, nDow )
   *-------------------------------------------------------------------
   *-- Programmer..: David Frankenbach (CIS: 72147,2635)
   *-- Date........: 06/20/1993
   *-- Notes.......: Returns number of workdays in a stub period of 
   *--               0 - 6 days beginning on the day of the week 
   *--               indicated.  Does not consider holidays, treats 
   *--               workdays as Monday thru Friday.
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/20/1993 - original function.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.XWorked(<nXtra>,<nDow>)
   *-- Example.....: _app.Holiday.XWorked( 5, 3 )
   *-- Returns.....: Number of workdays in the period, or -1 for error.
   *-- Parameters..: nXtra     -- days in the stub period.
   *--               nDoW      -- dow() of first day of the stub 
   *--                            period.
   *-------------------------------------------------------------------
      
      if nXtra < 0 .or. nXtra > 6 .or. nDoW < 1 .or. nDoW > 7
         RETURN -1
      endif
      
      if ( nDoW = 1 )
         RETURN max( ( nXtra - 1 ), 0 )
      else
         RETURN min( nXtra, max( 7 - nDoW, 0 ) ) ;
                     + max( nXtra - 9 + nDoW, 0 )
      endif
      
   RETURN -1
   *-- EoF: XWorked()

   *********************************************************
   *** The following methods use the table: MAKEHOLS.DBF ***
   *** And/or a table HOLSyyyy.DBF (year)                ***
   *********************************************************

   FUNCTION AddaHol( dDate, nYear, cHolname )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 06/26/1993
   *-- Notes.......: Adds a date to Holstemp if in correct year and not
   *--               already included.  Requires a file alias Holstemp 
   *--               in order Holdate in one work area and a file alias
   *--               Makehols in another, ordinarily current, work 
   *--               area. If a holiday is already included, replaces 
   *--               the name. This so that if Makehols() has already 
   *--               included the Sundays, for example, but Easter or 
   *--               a Jewish holiday which is not transferred falls 
   *--               on Sunday, it will be included in Makehols.dbf 
   *--               under the (last) name rather than by the name of 
   *--               the day of the week.  This makes it easier to see 
   *--               in HolsYYYY that it has not been omitted by 
   *--               mistake.
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/26/1993 - original function.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: None
   *-- Usage.......: holiday.AddaHol(<dDate>,<nYr>,<cHolname>)
   *-- Example.....: ? _app.Holiday.AddaHol( {12/25/93}, 1993, ;
   *--                                     "Christmas")
   *-- Returns.....: .T. if added, or .F. if not added ( duplicate or 
   *--               wrong year)
   *-- Parameters..: dDate     -- date to add
   *--               nYr       -- year in which it must fall, four 
   *--                            digits
   *--               cHolname  -- name of the holiday
   *-------------------------------------------------------------------
      
      local lReturn
      
      lReturn = .F.
      if year( dDate ) = nYr
         select Holstemp
         seek dDate
         if .not. found()
            append blank
            replace Holdate with dDate, ;
                    Holname with cHolname
            lReturn = .T.
         else
            replace Holname with cHolname
         endif
         select Makehols
      endif
   
   RETURN lReturn
   *-- EoF: AddaHol()

   FUNCTION MakeHols( nYear )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 04/22/1994
   *-- Notes.......: Makes a .dbf file of name "HolsYYYY.dbf" if the 
   *--               file Makehols.dbf is found, where "YYYY" in the 
   *--               name will be the four digits of the year.  The 
   *--               resulting file will contain a record for each 
   *--               non-working day of the year and will be sorted 
   *--               in order of date and tagged by date tag "HolDate".
   *--               Returns .T. if successful, or .F.  If the file 
   *--               "HolsYYYY.dbf" already exists, it will be rebuilt 
   *--               only if older than "Makehols.dbf." This function 
   *--               uses a .dbf called "Makehols.dbf".  If it cannot 
   *--               be found, does not exist, the function fails.  
   *--               Makehols.dbf may or may not be provided with
   *--               this library.  Its structure is:
   *--                       HOLTYPE   C  1   N   Type of holiday
   *--                       FIRSTDAY  C  6   N   date within the year*
   *--                       NUMDOW    N  1 0 N   dow()
   *--                       XFER      C  1   N   Transfer code
   *--                       XTEND     C  1   N   Extendability code
   *--                       HOLNAME   C 40   N   Name of the holiday
   *--               *The FIRSTDAY field is used for other purposes for
   *--               some holiday types.  See below
   *--               Values acceptable in the Makehols.dbf fields are:
   *--                 Holtype:  W = weekly holiday, e.g. Sunday
   *--                           X = fixed holiday, e.g. Xmas
   *--                           F = floating holiday, e.g. Labor Day
   *--                           J = Jewish holiday
   *--                           E = Easter or day calculated from it
   *--                 FirstDay: for W types, ignored
   *--                           for X types, the date "MM/DD/"
   *--                           for F types, first date on which
   *--                                 it can occur, "MM/DD/", or the
   *--                                 letter to pass to the Holidays()
   *--                                 function if supported in it.
   *--                           for J types, the month and day in the
   *--                                 Hebrew calendar, such as "01/02"
   *--                                 for the second day of Rosh
   *--                                 Hashanah.  Dates that occur
   *--                                 twice in the civil year will be
   *--                                 included twice.  The month is
   *--                                 understood as the month in a
   *--                                 common year, Nisan being 7.  If
   *--                                 month 6, Adar, is specified, the
   *--                                 holiday will be that date in
   *--                                 Second Adar in leap years.
   *--                                 To specify a date in I Adar of
   *--                                 a leap year, use month "00".
   *--                           for E types, the val() of this field
   *--                                 will be used as a displacement
   *--                                 from Easter, such as "-46" for
   *--                                 Ash Wednesday.
   *--                 NumDow:   dow() value of the holiday.  Needed
   *--                           for W types, and for F types if
   *--                           Firstday has a nonzero val(), ignored
   *--                           for others.
   *--                 XFer:     "F", "B", "Y" or other, meaning that
   *--                           if the holiday falls on a Weekly-type
   *--                           non-working day, transfer it:
   *--                                  F = Forward,
   *--                                  B = Backward,
   *--                                  Y = Yes, either way, or
   *--                              other = not at all,
   *--                           to the nearest working day in that
   *--                           direction.  If "Y" is specified, a
   *--                           transfer date ahead will be preferred
   *--                           to one the same number of days back.
   *--                 Xtend:    Same characters as Xfer, but here they
   *--                           indicate whether the holiday is to be
   *--                           extended to cover the intervening day
   *--                           if it falls one day away from a weekly
   *--                           holiday ( example--the Friday after
   *--                           Thanksgiving for some schools ).  Mark
   *--                           Thanksgiving with "F" or "Y" to have
   *--                           the Friday treated as a holiday if
   *--                           it would otherwise be the only working
   *--                           day between the major holiday and the
   *--                           weekend.
   *--
   *--                 Makehols.dbf must exist and must have its
   *--                 holidays entered properly for this function to
   *--                 succeed.  Since changes in the specification of
   *--                 holidays applicable to an organization are rare,
   *--                 no "front end" to facilitate changing the .dbf
   *--                 is provided.
   *--
   *--                 Each HolsYYYY.dbf file created by this function
   *--                 will have the following structure:
   *--                         Holdate     D    8       Y
   *--                         Holname     C   40       N
   *--                 with one record for each nonworking day,
   *--                 including each occurrence of the "W" types such
   *--                 as Sundays.  Temporary files named Holsstru.dbf,
   *--                 Holstemp.dbf and Holstemp.mdx are also created
   *--                 and later deleted.
   *--
   *--               WARNING: This function works properly only if the
   *--               date setting is AMERICAN when it is called. It has 
   *--               not been changed because if you want to use it for 
   *--               dates in other formats, you will probably also 
   *--               wish to change the manner in which the month and 
   *--               day of fixed and floating holidays are stored in
   *--               Makehols.dbf for ease of reading it. Comments are 
   *--               given for lines you may need to change.
   *--
   *-- Written for.: dBASE IV, 1.5 or higher.
   *-- Rev. History: 06/19/1993 - original version.
   *--               08/03/1993 - Jay Parsons - retained active .dbf,
   *--                            conformed to Civildate()
   *--               04/22/1994 - Jay Parsons - added note about date 
   *--                            set(), comments for affected lines
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: time.Time2Sec()
   *--               holiday.EasterDay()
   *--               hebrew.CivilDate()
   *--               holiday.HolXFer()
   *--               holiday.AddaHol()
   *-- Called by...: Any
   *-- Usage.......: MakeHols(<nYear>)
   *-- Example.....: MakeHols( 1993 )
   *-- Returns.....: .T. if successful, or .F.
   *-- Parameters..: nYear         -- the civil year of interest
   *-------------------------------------------------------------------
      
      local nYr, cAlias, d1, d2, cName, cHoltype
      local cWeekly, lOk, nMo, cTemp, cFirst

      *  check for existence of main file
      if .not. file( "MakeHols.dbf" )
         RETURN .F.
      endif
      
      *  convert year to four digits
      nYr = nYear
      if nYr < 100
         nYr = nYr + 100 * int( year( date() ) / 100 )
      endif
      
      *  save current work area
      cAlias = Alias()
      
      * check date and time stamps.  Return if up to date or erase
      d1 = fdate( "Makehols.dbf" )
      cName = "Hols" + str( nYr, 4 )
      if file( cName + ".dbf" )
         d2 = fdate( cName + ".dbf" )
         if d2 > d1 .or. ;
             ( d2 = d1 .and. ;
             this.Time.Time2Sec( ftime( cName + ".dbf" ) ) ;
             > this.Time.Time2Sec( ftime( "Makehols.dbf" ) ) )
            RETURN .T.
         else
            delete file ( cName + ".dbf" )
         endif
      endif
      select select()
      if file( "Holstemp.dbf" )
         use Holstemp order Holdate excl
         zap
      else
         use Makehols
         copy to Holsstru structure extended
         use Holsstru excl
         zap
         append blank
         replace FIELD_NAME with "Holdate", ;
                 FIELD_TYPE with "D", FIELD_LEN with 8, ;
                 FIELD_DEC with 0, FIELD_IDX with "Y"
         append blank
         replace FIELD_NAME with "Holname", ;
                 FIELD_TYPE with "C", FIELD_LEN with 40, ;
                 FIELD_DEC with 0, FIELD_IDX with "N"
         create HolsTemp from HolsStru
         index on Holdate tag Holdate
         delete file HolsStru.dbf
      endif
      
      *--  now add the holidays to HolsTemp
      select select()
      use Makehols
      *--  MakeHols may be in no order, so start with weekly holidays
      *--  This line needs change to conform to date set()
      d1 = ctod( "01/01/" + str( nYr, 4 ) )
      cWeekly = ""
      scan for upper( Holtype ) = "W"
         if upper( str( NumDow, 1 ) ) $ cWeekly
            loop
         else
            cWeekly = cWeekly + upper( str( NumDow, 1 ) )
         endif
         d2 = d1 + mod( NumDow + 7 - dow( d1 ), 7 )
         select Holstemp
         lOk = .T.
         do while lOk
            lOk = holiday.AddaHol( d2, nYr, cDow( d2 ) )
            d2 = d2 + 7
         enddo
         select Makehols
      endscan

      *--  then the other holidays
      scan for upper( Holtype ) # "W"
         cHoltype = upper( Holtype )
         cFirst = trim( Firstday ) ;
                     + iif( len( trim( Firstday ) ) = 5, "/","" )
         do case
            * fixed holidays - just transfer and add
            case cHoltype = "X"
               * line below may have to be changed to match date set()
               d1 = ctod( cFirst + str( nYr,4 ) )
               d2 = CLASS::HolXfer( d1, cWeekly, upper( Xfer ) )
               lOk = CLASS::AddaHol( d2, nYr, Holname )
            * floating holidays - use Holiday() or its
            * algorithm to find the date, transfer and add
            case cHoltype = "F"
               if val( cFirst ) = 0
                  d1 = CLASS::Holiday( nYr, cFirst )
               else
                  d2 = d1 + 7 - NumDow
                  d1 = d2 - dow( d2 ) + NumDow
               endif
               d2 = CLASS::HolXfer( d1, str( NumDow, 1 ), ;
                                     upper( Xfer ) )
               lOk = CLASS::AddaHol( d2, nYr, Holname )
            * Easter and dates referred to it - call
            * Easterday() to find the date, xfer and add
            case cHoltype = "E"
               d1 = CLASS::EasterDay( nYr ) + val( cFirst )
               d2 = CLASS::HolXfer( d1, cWeekly, upper( Xfer ) )
               lOk = CLASS::AddaHol( d2, nYr, Holname )
            * Jewish holidays - convert to civil date,
            * transfer and add.  Some dates, although none
            * of the major holidays, may occur twice in a
            * civil year.
            case cHoltype = "J"
               nMo = val( cFirst )
               * first see if dates late in the Hebrew
               * year occurred early this year
               if nMo > 2 .or. nMo = 0
                  d2 = this.Hebrew.Civildate( cFirst + str(nYr+3760, 4))
                  if year( d2 ) # 0
                     lOk = CLASS::AddaHol( d2, nYr, Holname )
                  endif
               endif
               * then check to see if dates early in
               * the Hebrew year occurred late this yr
               if nMo > 0 .and. nMo < 5
                  d2 = this.Hebrew.Civildate( cFirst + str(nYr +3761,4))
                  if year( d2 ) # 0
                     lOk = CLASS::AddaHol( d2, nYr, Holname )
                  endif
               endif
         endcase

         * add days for extended holidays as needed
         lOk = .F.
         if upper( Xtend ) $ "FY" .and. ;
             str( dow( d2 + 2 ), 1 ) $ cWeekly .and. ;
             .not. str( dow( d2 + 1 ), 1 ) $ cWeekly
            lOk = CLASS::AddaHol( d2 + 1, nYr, ;
                           trim( Holname ) + " ext" )
         endif
         if upper( Xtend ) $ "BY" .and. ;
             str( dow( d2 - 2 ), 1 ) $ cWeekly .and. ;
             .not. ( lOk .or. str( dow( d2 - 1 ), 1 ) ;
              $ cWeekly )
            lOk = CLASS::AddaHol( d2 - 1, nYr, trim( Holname ) +;
                                  " ext" )
         endif
      endscan
      select Holstemp
      copy to ( cName ) with production
      use
      erase Holstemp.dbf
      erase Holstemp.mdx
      select Makehols
      use
      if "" # cAlias
         select ( cAlias )
      endif

   RETURN .T.
   *-- EoF: MakeHols()

   FUNCTION WorkDays( dDate1, dDate2 )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302) 
   *-- Date........: 08/02/1993
   *-- Notes.......: Returns workdays between two dates, excluding the
   *--               first but including the last.
   *--                     This function uses a .dbf named 
   *--               "HolsYYYY.dbf", with "YYYY" being the four digits 
   *--               of the year, for each calendar year of interest.  
   *--               Such a .dbf will be created, if possible, if it 
   *--               does not exist by the MakeHols() function included 
   *--               elsewhere in this file. The .dbf may be created 
   *--               otherwise; the only requirements are that its 
   *--               first field must be a date and that it be both 
   *--               sorted and ordered on that field tag "Holdate".  
   *--               It is suggested that it include a second field of
   *--               character type, length 40, for the name of the 
   *--               holiday.  The HolsYYYY .dbf must contain one 
   *--               record for each non-working day of the year,
   *--               including all Sundays and the like.
   *--                     Because this function calls MakeHols() to 
   *--               check the timestamp of MakeHols.dbf against that 
   *--               of HolsYYYY.dbf, Makehols.dbf must exist even if
   *--               HolsYYYY.dbf is up to date.
   *--                     This function may of course be written to 
   *--               use an array for the non-working days instead of a
   *--               .dbf. That will require considerably more memory 
   *--               but may increase speed of access depending on 
   *--               caching, at the price of requiring rebuilding/
   *--               reloading the array when needed.
   *-- Written for.: dBASE IV, 1.1 or higher.
   *-- Rev. History: 06/19/1993 - original function.
   *--               08/02/1993 - Jay Parsons - cNear and lOk made 
   *--                            private.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/03/1996 -- Tinkered with minimally to make part
   *--               of custom class. Ken Mayer
   *-- Calls.......: holiday.MakeHols()
   *--               date.IsLeap()
   *--               date.DoY()
   *-- Usage.......: holiday.WorkDays(<dDate1>,<dDate2>)
   *-- Example.....: _app.Holiday.WorkDays(date(),{08/27/94})
   *-- Returns.....: Number of workdays in the period between the 
   *--               dates, excluding the first and excluding the last,
   *--               or -1 for error.
   *-- Parameters..: dDate1, dDate2 -- dates limiting the period.
   *-------------------------------------------------------------------
      
      local dStart, dEnd, nWork, nSt, cHols, cNear, lOk

      * deal with improper arguments
      if type("dDate1") # "D" .or. type("dDate2") # "D"
         RETURN -1
      endif
      
      * and reversal of order
      dStart = min( dDate1, dDate2 )
      dEnd = max( dDate1, dDate2 )
      
      nWork = 0
      if .not. file( "MAKEHOLS.DBF" )
         RETURN -1
      endif
      cNear = set("NEAR")
      set near on
      
      nYr = year( dStart )
      do while nYr <= year( dEnd )
         lOk = CLASS::MakeHols( nYr )
         if .not. lOk
            set near &cNear.
            RETURN -1
         endif
         use "HOLS" + str( nYr, 4 ) order HolDate ;
             alias Holsfile in select()
         * Add workdays in whole year, except final year
         if nYr # year( dEnd )
            nWork = nWork ;
                       + iif( this.Date.IsLeap( nYr ), 366, 365 ) ;
                        - reccount( "Holsfile" )
         else
            * for ending year, add earlier days - holidays
            nWork = nWork + this.Date.DoY( dEnd )
            seek dEnd
            nWork = nWork - iif( eof(), reccount(), ;
                          recno() - iif( found(), 0, 1 ) )
         endif
         * For starting year, remove earlier days - holidays
         if nYr = year( dStart )
            nWork = nWork - this.Date.DoY( dStart )
            seek dStart
            nWork = nWork + iif( eof(), reccount(), ;
                       recno() - iif( found(), 0, 1 ) )
         endif
         nYr = nYr + 1
         use
      enddo
      set near &cNear.

   RETURN nWork
   *-- EoF: WorkDays()

ENDCLASS
*-- End of Class: HOLIDAY

#INCLUDE <dUFLP.H>

*-----------------------------------------------------------------------
*-- End of File: HOLIDAY.CC
*-----------------------------------------------------------------------