*-- File: MISC.CC

CLASS Misc
*----------------------------------------------------------------------
*-- Description:
*
*   Miscellaneous methods that did not fit into any other custom
*   classes in this library.
*
*-- Programmers:
*
*   Ken Mayer
*   Jim Sare
*   Angus Scott-Fleming
*   Jay Parsons
*
*-- History: 
*
*   12/01/1996 -- original version of these routines as a class.
*
*-- Instantiation:
*
*   set procedure to misc.cc additive
*   _app.Misc = new Misc()
*
*   Typical usage would be to call individual methods from here,
*   such as:
*
*     ? _app.Misc.method()
*
*-- Methods:
*     Note, more details are in the headers for each method, including
*     a listing of parameters, and what they mean, as well as the
*     complete history (as we know it) for each method.
*
*--    Init()   
*        handles any "initialization" necessary, is handled
*        when a new instance of the Misc class is created.
*
*--    Release() 
*        releases this class and the string class if it wasn't
*        instantiated previously
*
*--    ActivateApp()
*        Activates another Windows software package
*
*--    Alarm()
*        Plays bell in a warbling fashion until a key is pressed
*
*--    DialUp()
*        Dials a number ...
*
*-- Dependencies:
*
*     STRINGEX.CC
*     DUFLP.H
*----------------------------------------------------------------------

   class::Init()   && initialize

   FUNCTION Init
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/09/1996
   *-- Notes.......: Ensures that routines needed are available
   *--               It is required that this be used (see instructions
   *--               above) for this class to function properly.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/09/1996 -- Original Code
   *-- Calls.......: string.cc
   *-- Usage.......: convert.Init()
   *-- Example.....: _app.Convert.Init()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------
   
      PROTECT StringProc
      
      if .not. SetProc( "STRINGEX.CC" )
         set procedure to stringex.cc additive
         this.StringProc = .t.
      else
         this.StringProc = .f.
      endif
      this.String        = new stringex()
      this.String.parent = this
      
   RETURN NULL
   *-- EoM: Init()
   
   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: convert.Release()
   *-- Example.....: _app.Convert.Release()
   *--               _app.Convert = NULL  && not required, good idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      if this.StringProc
         close procedure stringex.cc
      endif
      release object this
   
   RETURN NULL
   *-- EoM: Release()
  
   PROCEDURE ActivateApp(cAppName, cWinText)
   *-------------------------------------------------------------------
   *-- Programmer..: Jim Sare (CIS: 71333,1030)
   *-- Date........: 10/23/1994
   *-- Notes.......: Use to start other applications in Windows. if the
   *--               application is already started, will switch to the
   *--               application.
   *-- Written for.: dBASE for Windows version 5.0
   *-- Rev. History: 10/23/1994 -- Original
   *--               12/01/1996 -- Minor mods for this class
   *-- Calls.......: None
   *-- Usage.......: misc.ActivateApp(cAppName, cWinText)
   *-- Example.....: _app.ActivateApp("WinWord.EXE", "Microsoft Word")
   *-- Returns.....: NULL
   *-- Parameters..: cAppName = The EXE filename. Not case sensitive.
   *--               May or may not include fully qualified path.
   *--               cWinText = The window title bar text. Not case
   *--               sensitive. Will accept partial title text starting
   *--               from the first position in actual text.
   *-------------------------------------------------------------------

      if type("BringWindowToTop") <> "FP"
         extern CVOID BringWindowToTop(CHANDLE) USER.EXE
      endif
      if type("GetDesktopWindow") <> "FP"
         extern CHANDLE GetDesktopWindow(CVOID) USER.EXE
      endif
      if type("GetWindow") <> "FP"
         extern CHANDLE GetWindow(CHANDLE, CINT) USER.EXE
      endif
      if type("GetWindowText") <> "FP"
         extern CINT GetWindowText(CHANDLE, CSTRING, CINT) USER.EXE
      endif
      if type("IsIconic") <> "FP"
         extern CLOGICAL IsIconic(CHANDLE) USER.EXE
      endif
      if type("ShowWindow") <> "FP"
         extern CLOGICAL  ShowWindow(CHANDLE, CINT) USER.EXE
      endif

      #define GW_CHILD     5
      #define GW_HWNDNEXT  2
      #define SW_RESTORE   9

      ThehWND = GetDesktopWindow()
      ThehWND = GetWindow(ThehWND, GW_CHILD)
      do while ThehWND > 0
         buff = SPACE(81)
         GetWindowText(ThehWND, buff, 80)
         if left( upper( this.String.alltrim(buff) ) , len(cWinText) );
             == upper(cWinText)
            exit
         endif
         ThehWND = GetWindow(ThehWND, GW_HWNDNEXT)
      enddo
      if ThehWND > 0
         if IsIconic(ThehWND)
            ShowWindow(ThehWND, SW_RESTORE)
         endif
         BringWindowToTop(ThehWND)
      else
         run(.t.,cAppName)
      endif
      
   RETURN NULL
   *-- EoM: AppActivate

   FUNCTION Alarm( nRings )
   *-------------------------------------------------------------------
   *-- Programmer..: Angus Scott-Fleming
   *-- Date........: 10/05/1993
   *-- Note........: Ring my chimes in a warbling pattern until a key 
   *--               is pressed
   *-- Written for.: dBASE IV 1.5+
   *-- Rev. History: 10/05/1993 -- Original
   *--               04/15/1994 -- Updated to handle keypress to exit
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *-- Calls.......: None
   *-- Usage.......: ? misc.Alarm()
   *-- Example.....: lDummy = Inform("Hi!") .and. _app.Misc.Alarm()
   *-- Returns.....: NULL
   *-- Parameters..: <optional>: nRings - how many times to warble?
   *-------------------------------------------------------------------
     
      local lBellOff, nTimesRung, nLastKey, lConsOn
      if pcount() < 1
         nRings = 10
      endif
      store 0 to nTimesRung, nLastKey
      lConsOn = set("console")="ON"
      set console on
      lBellOff = (set("BELL") = "OFF")
      set bell on
      do while nTimesRung < nRings .and. nLastKey = 0
         set bell to 400,10
         ?? chr(7)
         set bell to 1200,10
         ?? chr(7)
         nLastKey = inkey()
         nTimesRung = nTimesRung + 1
      enddo
      * reset the bell
      if lBellOff
         set bell off
      endif
      set bell to
      if .not. lConsOn
         set console off
      endif

   RETURN NULL
   *-- EoM: Alarm

   FUNCTION DialUp( cPhoneNo )
   *-------------------------------------------------------------------
   *-- Programmer..: Jay Parsons (CIS: 72662,1302)
   *-- Date........: 06/17/1992
   *-- Notes.......: Dial the supplied telephone number.  Returns .F. 
   *                 for error. This is not a full communications 
   *                 routine. It is designed to be used to place voice
   *                 telephone calls, with the user picking up the 
   *                 handset after using this function to dial.
   *--
   *--               This will work only with a modem using the standard 
   *--               Hayes commands, and only if the port has already 
   *--               been set to the desired baud rate, etc., by the DOS 
   *--               MODE command or otherwise.  If the port and dialing 
   *--               method are not constant for the application, rewrite 
   *--               the function to accept them as additional parameters.
   *--
   *-- Written for.: dBASE IV, 1.1, 1.5
   *-- Rev. History: 03/01/1992 - original function.
   *--               04/01/1992 - Jay Parsons - modified for Version 1.5.
   *--               04/03/1992 - Jay Parsons - ferror() call added.
   *--               06/17/1992 - Jay Parsons - 1.1 version changed to use
   *--                              SET PRINTER TO Device rather than .bin.
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *-- Calls.......: stringex.StrpBrk()
   *-- Usage.......: misc.DialUp(<cPhoneNo>)
   *-- Example.....: x = DialUp( "555-1212" )
   *-- Returns.....: Logical (connect made or not)
   *-- Parameters..: cPhoneNo = Phone number to dial
   *-------------------------------------------------------------------

      local cNumber, cPort, cDialtype, cCallarg, nTemp, nHandle,;
            cString, lResult
      cPort = "Com2"          && specify Com1 or Com2 as required 
      cDialtype = "Tone"      && specify Tone or Pulse ( rotary ) dialing
      cNumber = cPhoneNo
      if type( "cPhoneno" ) $ "NF"
         cNumber = ltrim( str( cPhoneNo ) )
      else
         do while .t.
            nTemp = this.String.StrpBrk( cNumber, " ()-" )
            if nTemp = 0
               exit
            endif
            cNumber = stuff( cNumber, nTemp, 1, "" )
         enddo
      endif
      cString = "ATD" + upper( left( cDialtype, 1 )) + cNumber + chr(13)
      nHandle = fopen( cPort, "w" )
      if ferror() # 0
         RETURN .F.
      endif
      lResult = (fwrite( nHandle, cString) = len( cString ))
      fclose( m->nHandle )
   
   RETURN lResult
   *-- EoM: DialUp()


ENDCLASS
*-- End of Class: MISC

#INCLUDE <dUFLP.H>

*-----------------------------------------------------------------------
*-- End of File: MISC.CC
*-----------------------------------------------------------------------