*-- File: SPEEDTIP.CC

*----------------------------------------------------------------------
*-- Description:
*
*      This file contains a set of speedtip enabled objects that do
*      not, in standard Visual dBASE objects, have speedtips. This is
*      handy for those situations where you wish you had a speedtip
*      for that combobox ... (or whatever).
*      Examples of these may be seen on SPEEDTIP.WFM
*
*-- Programmers: 
*
*      Romain Streiff
*      Ken Mayer
*
*-- History: 
*
*      03/28/1996 -- original version of these routines.
*      01/19/1997 -- modifications (minor) to fit standard dUFLP
*                    comments, etc.
*
*-- Instantiation: 
*
*      set procedure to speedtip.cc additive
*      * drop an instance of these new custom controls onto a form and
*      * use as normal.
*
*-- Notes:
*
*      All the controls use the _same_ speedtip text control to save 
*      resources, it will change only the content and the position. 
*      All controls call the same proc OnMouseMove in the SPEEDTIP CLASS 
*      at the bottom of this file.
*
*-- Classes:
*
*--    SPEEDTIP_SPINBOX
*--    SPEEDTIP_EDITOR
*--    SPEEDTIP_LISTBOX
*--    SPEEDTIP_COMBOBOX
*--    SPEEDTIP_TEXT
*--    SPEEDTIP_TABBOX
*--    SPEEDTIP_BROWSE
*--    SPEEDTIP_IMAGE
*--    SPEEDTIP_SCROLLBAR
*--    SPEEDTIP_RECTANGLE
*--    SPEEDTIP (the work-horse ...)
*
*-- Properties:
*
*      form.SpeedTip
*      form.SpeedTip.Timer
*
*----------------------------------------------------------------------

CLASS speedtip_spinbox(FormObj,Name) OF SPINBOX(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
     
ENDCLASS
*-- End of Class: SPEEDTIP_SPINBOX

CLASS speedtip_editor(FormObj,Name) OF EDITOR(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_EDITOR

CLASS speedtip_listbox(FormObj,Name) OF LISTBOX(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_LISTBOX

CLASS speedtip_combobox(FormObj,Name) OF COMBOBOX(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_COMBOBOX

CLASS speedtip_text(FormObj,Name) OF TEXT(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_TEXT
 
CLASS speedtip_tabbox(FormObj,Name) OF TABBOX(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_TABBOX

CLASS speedtip_browse(FormObj,Name) OF BROWSE(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_BROWSE
 
CLASS speedtip_image(FormObj,Name) OF IMAGE(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_IMAGE 

CLASS speedtip_scrollbar(FormObj,Name) OF SCROLLBAR(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_SCROLLBAR
 
CLASS speedtip_rectangle(FormObj,Name) OF RECTANGLE(FormObj,Name) Custom
      Protect OnMousemove
      This.OnMouseMove = SPEEDTIP::ONMOUSEMOVE
ENDCLASS
*-- End of Class: SPEEDTIP_RECTANGLE
 

CLASS SPEEDTIP
 
   Procedure OnMouseMove(flags, col, row)
      *-- The following gets only executed the first time a user moves over a
      *-- Speedtip Custom Control
      if type("Form.speedtip")="U"  

        Form.speedtip=new text(Form)
        Form.speedtip.text = ""
        Form.speedtip.pageno=0
        Form.speedtip.border=.t.
        Form.speedtip.oldstyle=.t.
        Form.speedtip.alignment=4
        Form.speedtip.left=-300
        Form.speedtip.fontbold=.f.
        Form.speedtip.visible=.f.
        Form.speedtip.ColorNormal = "WindowText/0xb9ffff"
        Form.speedtip.height = 1
        *--When mouse is over the speedtip itself, disable it because you are 
        *--no more over the custom control
        Form.speedtip.onmousemove={;form.speedtip.visible=.f.}
        *--Create timer for the display delay
        Form.speedtip.timer=new timer()
        Form.speedtip.timer.oParent=Form.speedtip
        Form.speedtip.timer.interval=1
        Form.speedtip.timer.ontimer={;this.oparent.visible=.t.;this.enabled=.f.}

        Form.speedtip.oControl=""
        *-- set FORM onMousemove to disable speedtips
        if empty(Form.OnMouseMove)
           Form.OnMouseMove={;Form.speedtip.visible=.f.}
        else
           Form.sOnMouseMove=   Form.OnMouseMove
           Form.OnMouseMove={;Form.speedtip.visible=.f.;Form.sOnMouseMove()}
        endif  
      endif

   *--show speedtip if not already visible
     if Form.speedtip.visible=.f. .or. form.speedtip.ocontrol # this.name
        *-- the control must have a custom property called 'Speedtip' set in
        *-- its ONOPEN event
        if type("this.speedtip")=="C" 
          if .not. "TABBOX" $ this.classname
          Form.speedtip.Top = this.top+this.height+0.2
          else
          Form.speedtip.Top = this.top-1
          endif
          
          Form.speedtip.Left = this.left
          Form.speedtip.text=this.speedtip
          Form.speedtip.Width = Form.speedtip.gettextextent(this.speedtip)+1          
          Form.speedtip.timer.enabled=.t.
          Form.speedtip.oControl=this.name
        endif
      endif

ENDCLASS
*-- End of Class: SPEEDTIP

*-----------------------------------------------------------------------
*-- End of File: SPEEDTIP.CC
*-----------------------------------------------------------------------

