*-- File: TABLE.CC

CLASS Table
*----------------------------------------------------------------------
*-- Description:
*
*      This class contains a set of record-processing routines.
*
*-- Programmers: 
*
*      Ken Mayer
*      Bowen Moursund (CIS: 72662,436)
*      Rose Vines (CIS: 100026,3153)
*
*-- History: 
*
*      12/02/1996 -- original version of these routines as a class.
*
*-- Instantiation: 
*
*      set procedure to table.cc additive
*      _app.Table = new Table()
*
*-- Methods:
*      More details on each of these is contained in the commentary for
*      each method ...
*   
*--    Init()
*        Initializes classes necessary
*
*--    Release()
*        Releases this object (and others) from memory
*
*--    DBFDir()
*        Creates or overwrites DBFDIR.DBF and fills with directory 
*        information
*
*--    DBFName()
*        Returns just data base name regardless of full path
*
*--    Delete()
*        Deletes a database, mdx, and dbt
*
*--    GetExcl()
*        Attempt to open a table exclusively on a LAN.
*
*--    LDConvert()
*        Changes Language Driver in table's header
*
*--    Rename()
*        Renames dbf, mdx, dbt file
*
*--    Used()
*        See if table/alias is in use
*
*-- Dependencies:
*
*      FILEEX.CC
*      INDEX.CC
*      DUFLP.H
*----------------------------------------------------------------------

   class::init()
   
   FUNCTION Init
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/10/1996
   *-- Notes.......: Ensures that routines needed are available
   *--               It is required that this be used (see instructions
   *--               above) for this class to function properly.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/10/1996 -- Original Code
   *-- Calls.......: FILEEX.CC, INDEX.CC
   *-- Usage.......: table.Init()
   *-- Example.....: _app.Table.Init()
   *-- Returns.....: Nul
   *-- Parameters..: None
   *-------------------------------------------------------------------

      PROTECT FileProc, IndexProc
      if .not. SetProc( "FILEEX.CC" )
         set procedure to fileex.cc  additive
         this.FileProc = .t.
      else
         this.FileProc = .f.
      endif
      this.File        = new FileEx()
      this.File.parent = this
      
      if .not. SetProc( "INDEX.CC" )
         set procedure to index.cc   additive
         this.IndexProc = .t.
      else
         this.IndexProc = .f.
      endif
      this.Index        = new Index()
      this.Index.parent = this
      
   RETURN NULL

   FUNCTION Release
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer
   *-- Date........: 11/24/1996
   *-- Notes.......: Used to release this object from memory.
   *-- Written for.: Visual dBASE 5.5
   *-- Rev. History: 11/24/1996 -- Original Code
   *-- Calls.......: none
   *-- Usage.......: table.Release()
   *-- Example.....: _app.Table.Release()
   *--               _app.Table = NULL   && not required, but a good 
   *--                                   && idea
   *-- Returns.....: Null
   *-- Parameters..: None
   *-------------------------------------------------------------------

      if this.FileProc
         close procedure fileex.cc
      endif
      if this.IndexProc
         close procedure index.cc
      endif
      release object this

   RETURN NULL
   *-- EoM: Release()


   FUNCTION DBFDir(cSkeleton, cAttrib)
   *-------------------------------------------------------------------
   *-- Programmer..: Bowen Moursund (CIS: 72662,436)
   *-- Date........: 11/10/1996
   *-- Notes.......: DbfDir() creates or OVERWRITES DbfDir.Dbf, and
   *--               populates it with directory information gathered
   *--               with ADIR().
   *--
   *--                          Structure of DBFDIR.DBF
   *--                          -----------------------
   *--                          Field    Type  Len  Dec
   *--                          F_NAME      C   12    0
   *--                          F_SIZE      N   10    0
   *--                          F_DATE      D    8    0
   *--                          F_TIME      C    8    0
   *--                          F_ATTR      C    4    0
   *-- Written for.: dBASEWin, v5.0
   *-- Rev. History: 07/03/1992 -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: fileex.TempName()
   *-- Usage.......: table.DbfDir( <cSkeleton> [, <lcAttrib>] )
   *-- Examples....: nFiles = _app.Table.DbfDir( "*.dbf" )
   *--               nFiles = _app.Table.DbfDir( "c:\*.*", "HSD" )
   *-- Returns.....: Number of matching files found: reccount() of 
   *--               DbfDir
   *-- Parameters..: cSkeleton = directory path and file skeleton that 
   *--                           you want, as in ADIR(). Wildcards OK. 
   *--                           RTFM.
   *--               cAttrib   = Optional. DOS file attribute list, as
   *--                           in ADIR(). RTFM
   *-------------------------------------------------------------------
      
      local aDirArr, nNumFiles, cAlias, aDBFStruc, cTemp

      aDirArr = new Array(1)
      if pcount() = 2
         nNumFiles = adir(aDirArr, cSkeleton, cAttrib)
      else  && pcount() = 1
         nNumFiles = adir(aDirArr, cSkeleton)
     endif
     
     if nNumFiles > 0
        cAlias = alias()
        select select()
        if .not. file("dbfdir.dbf")
           cTemp = this.File.TempName("DBF")
           create cTemp structure extended
            aDBFStruc = new array(5,5)
            aDBFStruc[1,1] = "F_NAME"
            aDBFStruc[1,2] = "C"
            aDBFStruc[1,3] = 12
            aDBFStruc[1,4] = 0
            aDBFStruc[1,5] = "N"
            aDBFStruc[2,1] = "F_SIZE"
            aDBFStruc[2,2] = "N"
            aDBFStruc[2,3] = 10
            aDBFStruc[2,4] = 0
            aDBFStruc[2,5] = "N"
            aDBFStruc[3,1] = "F_DATE"
            aDBFStruc[3,2] = "D"
            aDBFStruc[3,3] = 8
            aDBFStruc[3,4] = 0
            aDBFStruc[3,5] = "N"
            aDBFStruc[4,1] = "F_TIME"
            aDBFStruc[4,2] = "C"
            aDBFStruc[4,3] = 8
            aDBFStruc[4,4] = 0
            aDBFStruc[4,5] = "N"
            aDBFStruc[5,1] = "F_ATTR"
            aDBFStruc[5,2] = "C"
            aDBFStruc[5,3] = 5
            aDBFStruc[5,4] = 0
            aDBFStruc[5,5] = "N"
            append from array aDBFStruc
            create dbfdir.dbf from cTemp
            erase cTemp
            use DBFDIR
        else
            use DBFDIR exclusive
            local lSafety
            lSafety = set("SAFETY") = "ON"
            set safety off
            zap
            if lSafety
                set safety on
            endif
         endif
         append from array aDirArr
         use
         if "" <> cAlias
            select (cAlias)
         endif
      endif

   RETURN nNumFiles
   *-- EoM: DBFDir()

   FUNCTION DBFName
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines (CIS: 100026,3153)
   *-- Date........: 11/10/1996
   *-- Notes.......: Strips the 8-character DBF filename out of the 
   *--               full pathname returned by the dbf() function. 
   *--               Works on the database in USE in the current 
   *--               workarea.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 04/26/1993  -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302                                     ,
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: table.DbfName()
   *-- Example.....: ? _app.Table.DbfName()
   *-- Returns.....: cName   =  8-character filename of DBF.
   *-- Parameters..: None
   *-------------------------------------------------------------------
      
      local cFullPath, cName
      
      cFullPath = set("FULLPATH")
      set fullpath off
      
      *-- Check if a database is open in the current workarea
      if "" <> dbf()
         *-- Strip the filename out of the full pathname
         cName = ( substr( dbf(), 3, at( ".", dbf() ) - 3 ) )
      else
         cName = ""
      endif
      
      set fullpath &cFullPath.
      
   RETURN (cName)
   *-- EoM: DBFName()

   FUNCTION Delete( DbfName )
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines (CIS: 100026,3153)
   *-- Date........: 11/18/1996
   *-- Notes.......: Deletes a database, its production index and its 
   *--               memo file (if there is one) in one fell swoop.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 04/26/1993  -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/18/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: table.Delete(<cDbfName>)
   *-- Example.....: _app.Table.Delete("FRED")
   *-- Returns.....: nError    - 0 if file deletion went okay
   *--                         - 1 if file to be deleted didn't exist.
   *-- Parameters..: cDbfName  - Name of the database you wish to 
   *--                           delete.
   *-------------------------------------------------------------------
      
      local cDbfName, cMdxName, cDbtName, nError
      
      cMdxName  = cDbfName + ".MDX"
      cDbtName  = cDbfName + ".DBT"
      cDbfName  = cDbfName + ".DBF"
      nError    = 0
      
      *-- Check database exists
      if file(cDbfName)
         *-- Delete database
         delete file (cDbfName)
         
         *-- Delete its MDX
         if file(cMdxName)
            delete file (cMdxName)
         endif
         
         *-- Delete its memo file if any
         if file(cDbtName)
            delete file (cDbtName)
         endif
         
      else
         
         nError = 1
          
      endif
      
   RETURN (nError)
   *-- EoM: Delete()

   FUNCTION GetExcl( cAlias )
   *-------------------------------------------------------------------
   *-- Programmer..: Keith G. Chuvala (CIS: 71333,2654)
   *-- Date........: 12/02/1996
   *-- Notes.......: Written to attempt to USE a file in EXCLusive
   *--               mode (on a LAN), locking out other users, so that 
   *--               your code will not give errors when PACKING
   *--               or performing some other operation that
   *--               requires EXCL use of a file.
   *-- Written for.: dBASE IV, Version 2.0
   *-- Rev. History: 07/25/1994 -- First version
   *--               08/02/1994 -- Added call to ALERT()
   *--               12/02/1996 -- For Windows, changed Alert()
   *--                             call to MsgBox()
   *-- Calls.......: None
   *-- Usage.......: GetExcl( <cAlias> )
   *-- Example.....: if GetExcl( "MyFile" )
   *--                  pack
   *--               else
   *--                  *-- error messsage
   *--               endif
   *-- Returns.....: Logical (.t. = successful)
   *-- Parameters..: cAlias = Table or Alias name to use EXCL
   *-- Side Effects: You may need to reset your ON ERROR handler
   *--               if you use this code (and are using your own
   *--               ON ERROR routine).
   *-------------------------------------------------------------------

      private nErrNum, lRetVal
      
      nErrNum = 0    && initialize
      lRetVal = .f.
      
      *-- if file is open in an area the user can
      *-- access:
      if select(m->cAlias) > 0
         use in (m->cAlias)
      endif
      
      *-- now set on error to deal with it if there's an error:
      on error nErrNum = error()        && store error number ...
      use (m->cAlias) excl in select()  && try to open exclusive
      if m->nErrNum = 0                 && if no error, we're cool
         select(m->cAlias)
         lRetVal = .t.
      else
         msgbox("File in Use by Another "+trim(m->cAlias)+" is in "+;
                "use by someone else on the LAN, try again later.",;
                "Alert",16)
      endif
      on error                          && reset error handler
      
   RETURN m->lRetVal
   *-- Eo<: GetExcl()

   FUNCTION LDConvert(cDBFName)
   *------------------------------------------------------------------
   *-- Programmer..: Bowen Moursund (CIS: 72662,436), Copyright 1994
   *-- Date........: 12/02/1996
   *-- Notes.......: Converts a mismatched table to the current global
   *--               IDAPI language driver. Compound index tags or 
   *--               index tags with expressions must be recreated 
   *--               manually.
   *--
   *--               WARNINGS!! Back up your tables BEFORE running this
   *--               routine!! This routine will overwrite DBFs named
   *--               LDCNVRT1.TMP and LDCNVRT2.TMP
   *-- Written for.: dBASE for Windows v5.0
   *-- Rev. History: 04/15/1995 -- Original
   *--               12/02/1996 -- Modified for this library
   *-- Calls.......: None
   *-- Usage.......: table.LDConvert( <cDBFName> )
   *-- Example.....: _app.table.LDConvert( "MYTABLE.DBF" )
   *-- Returns.....: None
   *-- Parameters..: cDBFName = The name of the table to be converted
   *-------------------------------------------------------------------

      set safety off
      set ldcheck off
      use (cDBFName)
      copy to LDCNVRT1.TMP structure extended
      use
      create LDCNVRT2.TMP from LDCNVRT1.TMP
      append from (cDBFName)
      copy to (cDBFName) with production
      use
      erase LDCNVRT1.TMP
      erase LDCNVRT2.TMP
      if file("LDCNVRT1.MDX")
         erase LDCNVRT1.MDX
      endif
      if file("LDCNVRT1.DBT")
         erase LDCNVRT1.DBT
      endif
      if file("LDCNVRT2.MDX")
         erase LDCNVRT2.MDX
      endif
      if file("LDCNVRT2.DBT")
         erase LDCNVRT2.DBT
      endif
      set safety on
      
   RETURN NULL
   *-- EoM: LDConvert

   FUNCTION Rename( cOldFile, cNewFile )
   *-------------------------------------------------------------------
   *-- Programmer..: Rose Vines (CIS: 100026,3153)
   *-- Date........: 11/18/1996
   *-- Notes.......: Renames a .DBF file and its production index and 
   *--               memo files (if they exist) and correctly updates 
   *--               the .MDX header. This is a variant of RenFile().
   *--               In this version, it doesn't matter whether the
   *--               database is open or closed when you call the UDF. 
   *--               If it is open, the
   *--
   *--               * current index order
   *--               * record pointer
   *--
   *--               will be restored after the renaming.
   *--               You must SET DBTRAP OFF before calling this UDF.
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 04/26/1993  -- Original
   *--               Fall, 1994 -- revised for dBASE 5.0 for Windows.
   *--                             Jay Parsons, 72662,1302
   *--               11/18/1996 -- Modified to work with this class.
   *-- Calls.......: index.MdxPoint() 
   *--               table.Used()
   *-- Usage.......: table.Rename(<cOldFile>, <cNewFile>)
   *-- Example.....: _app.Table.Rename("FRED","MARY")
   *-- Returns.....: nError   - 0 if renaming operation went okay.
   *--                          1 if file to be renamed didn't exist.
   *-- Parameters..: cOldFile - Current database name
   *--               cNewFile - New name for database
   *-------------------------------------------------------------------
      
      local cOldFile, cNewFile, lOpen, nError, nRec, cTag

      lOpen  = .F.
      nError = 0
      
      *-- Check if database exists
      if file(cOldFile + ".DBF")
         
         *-- If database is currently open, save record pointer
         *-- and index order
         if CLASS::Used( upper(cOldFile) )
            nRec   = recno()
            cTag   = tag()
            lOpen  = .T.
            use
         endif
         
         *-- Rename database
         rename cOldFile + ".DBF" to cNewFile + ".DBF"
         
         *-- Rename its MDX
         if file(cOldFile + ".MDX")
            rename cOldFile + ".MDX" to cNewFile + ".MDX"
            *-- Update the hard-coded database reference in the MDX 
            *-- header
            this.Index.MdxPoint(cNewFile,"")
         endif
         
         *-- Rename its memo file
         if file(cOldFile + ".DBT")
            rename cOldFile + ".DBT" to cNewFile + ".DBT"
         endif
         
         *-- If file was originally open, reopen it and restore its state
         if lOpen
            use (cNewFile)
            if "" <> cTag
               set order to (cTag)
            endif
            go nRec
         endif
         
      else
         
         nError = 1
         
      endif
      
   RETURN (nError)
   *-- EoM: Rename()

   FUNCTION Used( cFile )
   *-------------------------------------------------------------------
   *-- Programmer..: Ken Mayer 
   *-- Date........: 11/10/1996
   *-- Notes.......: Created because the picklist routine by Malcolm 
   *--               Rubel from DBA Magazine (11/91) calls a function 
   *--               that checks to see if a DBF file is open ...
   *-- Written for.: dBASE IV, 1.5
   *-- Rev. History: 05/15/1992 -- Original
   *--               02/08/1993 -- Discovered (thanks to Jay, and then
   *--               Malcolm) a much simpler way to do this ...
   *--               11/10/1996 -- Modified to work with this class.
   *-- Calls.......: None
   *-- Usage.......: _app.Table.Used("<cFile>")
   *-- Example.....: if _app.Table.used("Library")
   *--                  select library
   *--               else
   *--                  select select()
   *--                  use library
   *--               endif
   *-- Returns.....: Logical (.t. if file is in use, .f. if not)
   *-- Parameters..: cFile = file to check for
   *-------------------------------------------------------------------

   RETURN (select(cFile) # 0)
   *-- EoM: Used()

ENDCLASS
*-- End of Class: TABLE

#INCLUDE <DUFLP.H>

*-----------------------------------------------------------------------
*-- End of File: TABLE.CC
*-----------------------------------------------------------------------
